@@ -169,25 +169,6 @@ struct ProjectStatusDisplayData {
 
 impl ProjectStatusDisplayData {
   pub fn from_status(status: &tree::ProjectStatus) -> ProjectStatusDisplayData {
-    let ahead = if status.ahead != 0 {
-      Some(console::style(format!("↑{}", status.ahead)).green())
-    } else {
-      None
-    };
-
-    let behind = if status.behind != 0 {
-      Some(console::style(format!("↓{}", status.behind)).red())
-    } else {
-      None
-    };
-
-    let ahead_behind = match (ahead, behind) {
-      (Some(a), Some(b)) => format!(" [{} {}]", a, b),
-      (Some(a), None) => format!(" [{}]", a),
-      (None, Some(b)) => format!(" [{}]", b),
-      (None, None) => "".to_string(),
-    };
-
     let branch = match &status.branch {
       Some(branch) => BRANCH_STYLE.apply_to(format!("branch {}", branch)),
       None => console::style("no branch".to_string()).red(),
@@ -195,7 +176,7 @@ impl ProjectStatusDisplayData {
 
     ProjectStatusDisplayData {
       location: PROJECT_STYLE.apply_to(format!("project {}", status.path)).to_string(),
-      branch: format!("{}{}", branch, ahead_behind),
+      branch: format!("{}{}", branch, util::ahead_behind(status.ahead, status.behind)),
       top_commit: status.commit_summary.clone().unwrap_or_default(),
       files: status
         .files
@@ -350,7 +331,7 @@ fn cmd_import(config: Arc<Config>, pool: &mut Pool, target_path: Option<&str>, c
         .join(".repo")
         .join("project-objects")
         .join(project.clone() + ".git");
-      let depot_path = depot.objects_mirror(remote_config, Depot::apply_project_renames(remote_config, &project));
+      let depot_path = depot.objects_mirror(remote_config, &Depot::apply_project_renames(remote_config, &project));
 
       if !src_path.is_dir() {
         eprintln!("Skipping missing project: {}", project);
@@ -675,17 +656,34 @@ fn main() {
     }
   };
 
-  let mut pool = match matches.value_of("JOBS") {
-    Some(jobs) => {
-      if let Ok(jobs) = jobs.parse::<usize>() {
-        Pool::with_size(jobs)
+  let cmd = matches.subcommand();
+  let pool_size = matches
+    .value_of("JOBS")
+    .map(|job_str| {
+      if let Ok(jobs) = job_str.parse::<i32>() {
+        jobs
       } else {
-        fatal!("failed to parse jobs value: {}", jobs);
+        fatal!("failed to parse jobs value: {}", job_str);
       }
-    }
+    })
+    .or_else(|| {
+      // Command-specific override
+      config.parallelism.get(cmd.0).cloned().or_else(|| {
+        // Global override
+        config.parallelism.get("global").cloned()
+      })
+    })
+    .unwrap_or(0);
 
-    None => Pool::with_default_size(),
+  let num_cpus = num_cpus::get();
+  let pool_size = if pool_size == 0 {
+    num_cpus
+  } else if pool_size < 0 {
+    std::cmp::min(num_cpus, (-pool_size) as usize)
+  } else {
+    pool_size as usize
   };
+  let mut pool = Pool::with_size(pool_size);
 
   let update_checker = if config.update_check && isatty::stdout_isatty() {
     Some(UpdateChecker::fetch())
@@ -694,7 +692,7 @@ fn main() {
   };
 
   let result = || -> Result<i32, Error> {
-    match matches.subcommand() {
+    match cmd {
       ("init", Some(submatches)) => {
         let fetch = !submatches.is_present("LOCAL");
         cmd_clone(
